// doj library
// $Header: /code/doj/file.hpp,v 1.7 2005/07/05 17:07:51 doj Exp $

#ifndef FILE__HPP
#define FILE__HPP

// RAII wrapper for stdio FILE pointer
// from Bjarne Stroustrup "The C++ Programming Language"

#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string>

namespace doj
{

  class FILEp
  {
    FILE *fp_;
  public:
    FILEp(FILE *fp) : fp_(fp) { }
    FILEp(const char *filename, const char *mode) : fp_(fopen(filename, mode)) { }
    ~FILEp() { if(fp_) fclose(fp_); }

    operator FILE*() const { return fp_; }
    FILE* get() const { return fp_; }

    FILE* release() { FILE *temp=fp_; fp_=0; return temp; }

    FILEp& swap(FILEp& a)
    {
      FILE *t=fp_;
      fp_=a.fp_;
      a.fp_=t;
      return *this;
    }

    void clearerr() { assert(fp_); ::clearerr(fp_); }
    int eof() { assert(fp_); return feof(fp_); }
    int error() { assert(fp_); return ferror(fp_); }
    int fd() { assert(fp_); return fileno(fp_); }
    int read(void *ptr, size_t size, size_t nmemb) { assert(fp_); return fread(ptr, size, nmemb, fp_); }
    int write(const void *ptr, size_t size, size_t nmemb) { assert(fp_); return fwrite(ptr, size, nmemb, fp_); }
    int stat(struct stat *buf) { assert(fp_); return fstat(fd(), buf); }
    off_t tell() { assert(fp_); return ftello(fp_); }
    void rewind() { assert(fp_); ::rewind(fp_); }
    int seek(off_t offset, int whence) { assert(fp_); return fseeko(fp_, offset, whence); }

  private:
    FILEp();
    FILEp(const FILEp&);
    FILEp& operator=(const FILEp&);
  };

}

#endif
